#include "..\\MappingLib.h"
#include "..\\Glove.h"
#include "..\\Lynx.h"

void pause()
{
	while( !_kbhit() ); _getch();
}

/* set the valid range for glove fingers and robot joints.
   you can change the values to suit your device.
*/
void initRange( Glove& g, Lynx& l )
{
	Glove::GloveCalib calib;
	calib.min[0] = 60; calib.max[0] = 90;
	calib.min[1] = 35; calib.max[1] = 60;
	calib.min[2] = 20; calib.max[2] = 60;
	calib.min[3] = 20; calib.max[3] = 60;
	calib.min[4] = 20; calib.max[4] = 60;
	//printf("Press any key after you are ready ...\n");
	//pause();
	//printf("start calibrating ...\n" );
	//if (!g.calibrate(&calib)){
	//	printf("failed, aborting.\n\n");
	//	return;
	//} 
	//else
	//	printf("succeeded.\n\n");

	//printf("Calibration data gathered:\n\n");
	//for (int i = 0; i < 5; i++){
	//	printf("%f  <= (finger #%d) <= %f\n", calib.min[i], i + 1, calib.max[i] );
	//}
	//pause();
	g.setCalib( calib );
	
	Lynx::LynxRange armRange;
	armRange.min[0] = 0; armRange.max[0] = M_PI;
	armRange.min[1] = 0.3; armRange.max[1] = M_PI_4*3;
	armRange.min[2] = 0; armRange.max[2] = M_PI;
	armRange.min[3] = 0.8; armRange.max[3] = 2;
	armRange.min[4] = 0.5; armRange.max[4] = 0.8;
	l.setRange( armRange );
}

int main( void )
{
	Glove g;
	g.setComm( 1 );
	printf("Glove attempting to connect on COM %d ...\n", g.getComm() );
	if( !g.open() ){
		pause();
		return 0;
	}
	Lynx l;
	l.setComm(7);
	printf("Lynx attempting to connect on COM %d ...\n", l.getComm() );
	if( !l.open() ){
		pause();
		return 0;
	}

	initRange( g, l );
	Glove::GloveCalib calib = g.getCalib();
	Lynx::LynxRange lrange = l.getRange();

	/* read out the "full" range of glove and robot to initialize the control box
	   NOTE for glove, maximum is curled finger, minimum is extended finger
    */
	double fingerRange[5][2], jointRange[5][2];
	for( int i = 0; i<5; i++ ){
		fingerRange[i][0] = calib.min[i];
		fingerRange[i][1] = calib.max[i];
		jointRange[i][0] = lrange.min[i];
		jointRange[i][1] = lrange.max[i];
	}

	Glove::GloveData frame;
	Lynx::LynxPos pos;
	Lynx::LynxCmd c;

	int youBox = 3; // the index of the black box applied: 1-3
	char fname[30] = "take1.txt"; // name of the file to save recording
	FILE* fp;

	// light: get your black box, it's on sale!
	void* box = acquireCtrlBox( youBox );
	setCtrlBox( box, fingerRange, jointRange );
	fopen_s( &fp, fname, "w" );
 
	// camera: set initial state of the system
	printf("reading the initial state ...\n" );Sleep(100);
	g.getState( & frame );
	pos = l.getPos();
	for( int i=0; i<5; i++ ){
		c.joint_set[i] = true;
		c.joint_spd[i] = 1;
		c.joint_tgt[i] = pos.joint_pos[i];
	}
	kickoffCtrlBox( box, frame.finger_pos, c.joint_tgt );
	
	// action: start rolling
	printf("start running. press any key to stop ...\n" );// Sleep(100);
	double rfs[3] = {0.01, 0.01, 0.05};
	double freq = 0.001, readfreq = rfs[youBox-1];
	double cur_t = 0, counter = 0;
	while( !_kbhit() ){
		g.getState( &frame );
		passCtrlBox( box, frame.finger_pos, c.joint_tgt, cur_t );
		cur_t += freq;
		counter += freq;

		Sleep( DWORD(freq*1000+0.5) );
		if ( counter<readfreq ) continue;

		counter = 0;

		l.setCmd( c );
		fprintf_s( fp, "%f", cur_t );
		for( int i=0; i<5; i++ ){
			fprintf_s( fp, " %f", frame.finger_pos[i] );
		}
		for( int i=0; i<5; i++ ){
			fprintf_s( fp, " %f", c.joint_tgt[i] ); // the "ideal" target
		}
		fprintf( fp, "\n" );
	}
	_getch();

	printf( "Press any key to exit. \n" );
	pause();
	fclose( fp );
	releaseCtrlBox( box );
	return 1;
}
